@@ -1,15 +1,20 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2009-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2010 Daniel Harding <dharding@gmail.com>
-# Copyright (c) 2013-2016, 2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2013-2016, 2018-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2013-2014 Google, Inc.
 # Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
 # Copyright (c) 2016 Jared Garst <jgarst@users.noreply.github.com>
 # Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
-# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2017, 2019 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2017 rr- <rr-@sakuya.pl>
+# Copyright (c) 2018 Serhiy Storchaka <storchaka@gmail.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
 # Copyright (c) 2018 brendanator <brendan.maginnis@gmail.com>
 # Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2019 Alex Hall <alex.mojaki@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
@@ -21,8 +26,6 @@
 * :func:`dump` function return an internal representation of nodes found
   in the tree, useful for debugging or understanding the tree structure
 """
-import sys
-
 
 # pylint: disable=unused-argument
 
@@ -41,7 +44,7 @@ class AsStringVisitor:
 
     def _docs_dedent(self, doc):
         """Stop newlines in docs being indented by self._stmt_list"""
-        return '\n%s"""%s"""' % (self.indent, doc.replace("\n", DOC_NEWLINE))
+        return '\n{}"""{}"""'.format(self.indent, doc.replace("\n", DOC_NEWLINE))
 
     def _stmt_list(self, stmts, indent=True):
         """return a list of nodes to string"""
@@ -60,8 +63,8 @@ class AsStringVisitor:
 
     def _should_wrap(self, node, child, is_left):
         """Wrap child if:
-            - it has lower precedence
-            - same precedence with position opposite to associativity direction
+        - it has lower precedence
+        - same precedence with position opposite to associativity direction
         """
         node_precedence = node.op_precedence()
         child_precedence = child.op_precedence()
@@ -82,6 +85,15 @@ class AsStringVisitor:
 
     ## visit_<node> methods ###########################################
 
+    def visit_await(self, node):
+        return "await %s" % node.value.accept(self)
+
+    def visit_asyncwith(self, node):
+        return "async %s" % self.visit_with(node)
+
+    def visit_asyncfor(self, node):
+        return "async %s" % self.visit_for(node)
+
     def visit_arguments(self, node):
         """return an astroid.Function node as string"""
         return node.format_args()
@@ -93,7 +105,9 @@ class AsStringVisitor:
     def visit_assert(self, node):
         """return an astroid.Assert node as string"""
         if node.fail:
-            return "assert %s, %s" % (node.test.accept(self), node.fail.accept(self))
+            return "assert {}, {}".format(
+                node.test.accept(self), node.fail.accept(self)
+            )
         return "assert %s" % node.test.accept(self)
 
     def visit_assignname(self, node):
@@ -103,11 +117,13 @@ class AsStringVisitor:
     def visit_assign(self, node):
         """return an astroid.Assign node as string"""
         lhs = " = ".join(n.accept(self) for n in node.targets)
-        return "%s = %s" % (lhs, node.value.accept(self))
+        return "{} = {}".format(lhs, node.value.accept(self))
 
     def visit_augassign(self, node):
         """return an astroid.AugAssign node as string"""
-        return "%s %s %s" % (node.target.accept(self), node.op, node.value.accept(self))
+        return "{} {} {}".format(
+            node.target.accept(self), node.op, node.value.accept(self)
+        )
 
     def visit_annassign(self, node):
         """Return an astroid.AugAssign node as string"""
@@ -115,8 +131,8 @@ class AsStringVisitor:
         target = node.target.accept(self)
         annotation = node.annotation.accept(self)
         if node.value is None:
-            return "%s: %s" % (target, annotation)
-        return "%s: %s = %s" % (target, annotation, node.value.accept(self))
+            return f"{target}: {annotation}"
+        return "{}: {} = {}".format(target, annotation, node.value.accept(self))
 
     def visit_repr(self, node):
         """return an astroid.Repr node as string"""
@@ -127,9 +143,9 @@ class AsStringVisitor:
         left = self._precedence_parens(node, node.left)
         right = self._precedence_parens(node, node.right, is_left=False)
         if node.op == "**":
-            return "%s%s%s" % (left, node.op, right)
+            return f"{left}{node.op}{right}"
 
-        return "%s %s %s" % (left, node.op, right)
+        return f"{left} {node.op} {right}"
 
     def visit_boolop(self, node):
         """return an astroid.BoolOp node as string"""
@@ -150,47 +166,38 @@ class AsStringVisitor:
             keywords = []
 
         args.extend(keywords)
-        return "%s(%s)" % (expr_str, ", ".join(args))
+        return "{}({})".format(expr_str, ", ".join(args))
 
     def visit_classdef(self, node):
         """return an astroid.ClassDef node as string"""
         decorate = node.decorators.accept(self) if node.decorators else ""
-        bases = ", ".join(n.accept(self) for n in node.bases)
-        metaclass = node.metaclass()
-        if metaclass and not node.has_metaclass_hack():
-            if bases:
-                bases = "(%s, metaclass=%s)" % (bases, metaclass.name)
-            else:
-                bases = "(metaclass=%s)" % metaclass.name
-        else:
-            bases = "(%s)" % bases if bases else ""
+        args = [n.accept(self) for n in node.bases]
+        if node._metaclass and not node.has_metaclass_hack():
+            args.append("metaclass=" + node._metaclass.accept(self))
+        args += [n.accept(self) for n in node.keywords]
+        args = "(%s)" % ", ".join(args) if args else ""
         docs = self._docs_dedent(node.doc) if node.doc else ""
-        return "\n\n%sclass %s%s:%s\n%s\n" % (
-            decorate,
-            node.name,
-            bases,
-            docs,
-            self._stmt_list(node.body),
+        return "\n\n{}class {}{}:{}\n{}\n".format(
+            decorate, node.name, args, docs, self._stmt_list(node.body)
         )
 
     def visit_compare(self, node):
         """return an astroid.Compare node as string"""
         rhs_str = " ".join(
             [
-                "%s %s" % (op, self._precedence_parens(node, expr, is_left=False))
+                "{} {}".format(op, self._precedence_parens(node, expr, is_left=False))
                 for op, expr in node.ops
             ]
         )
-        return "%s %s" % (self._precedence_parens(node, node.left), rhs_str)
+        return "{} {}".format(self._precedence_parens(node, node.left), rhs_str)
 
     def visit_comprehension(self, node):
         """return an astroid.Comprehension node as string"""
         ifs = "".join(" if %s" % n.accept(self) for n in node.ifs)
-        return "for %s in %s%s" % (
-            node.target.accept(self),
-            node.iter.accept(self),
-            ifs,
+        generated = "for {} in {}{}".format(
+            node.target.accept(self), node.iter.accept(self), ifs
         )
+        return "{}{}".format("async " if node.is_async else "", generated)
 
     def visit_const(self, node):
         """return an astroid.Const node as string"""
@@ -230,14 +237,14 @@ class AsStringVisitor:
                 # It can only be a DictUnpack node.
                 yield key + value
             else:
-                yield "%s: %s" % (key, value)
+                yield f"{key}: {value}"
 
     def visit_dictunpack(self, node):
         return "**"
 
     def visit_dictcomp(self, node):
         """return an astroid.DictComp node as string"""
-        return "{%s: %s %s}" % (
+        return "{{{}: {} {}}}".format(
             node.key.accept(self),
             node.value.accept(self),
             " ".join(n.accept(self) for n in node.generators),
@@ -254,15 +261,14 @@ class AsStringVisitor:
     def visit_excepthandler(self, node):
         if node.type:
             if node.name:
-                excs = "except %s, %s" % (
-                    node.type.accept(self),
-                    node.name.accept(self),
+                excs = "except {} as {}".format(
+                    node.type.accept(self), node.name.accept(self)
                 )
             else:
                 excs = "except %s" % node.type.accept(self)
         else:
             excs = "except"
-        return "%s:\n%s" % (excs, self._stmt_list(node.body))
+        return "{}:\n{}".format(excs, self._stmt_list(node.body))
 
     def visit_ellipsis(self, node):
         """return an astroid.Ellipsis node as string"""
@@ -275,13 +281,15 @@ class AsStringVisitor:
     def visit_exec(self, node):
         """return an astroid.Exec node as string"""
         if node.locals:
-            return "exec %s in %s, %s" % (
+            return "exec {} in {}, {}".format(
                 node.expr.accept(self),
                 node.locals.accept(self),
                 node.globals.accept(self),
             )
         if node.globals:
-            return "exec %s in %s" % (node.expr.accept(self), node.globals.accept(self))
+            return "exec {} in {}".format(
+                node.expr.accept(self), node.globals.accept(self)
+            )
         return "exec %s" % node.expr.accept(self)
 
     def visit_extslice(self, node):
@@ -290,33 +298,66 @@ class AsStringVisitor:
 
     def visit_for(self, node):
         """return an astroid.For node as string"""
-        fors = "for %s in %s:\n%s" % (
-            node.target.accept(self),
-            node.iter.accept(self),
-            self._stmt_list(node.body),
+        fors = "for {} in {}:\n{}".format(
+            node.target.accept(self), node.iter.accept(self), self._stmt_list(node.body)
         )
         if node.orelse:
-            fors = "%s\nelse:\n%s" % (fors, self._stmt_list(node.orelse))
+            fors = "{}\nelse:\n{}".format(fors, self._stmt_list(node.orelse))
         return fors
 
     def visit_importfrom(self, node):
         """return an astroid.ImportFrom node as string"""
-        return "from %s import %s" % (
-            "." * (node.level or 0) + node.modname,
-            _import_string(node.names),
+        return "from {} import {}".format(
+            "." * (node.level or 0) + node.modname, _import_string(node.names)
         )
 
-    def visit_functiondef(self, node):
-        """return an astroid.Function node as string"""
+    def visit_joinedstr(self, node):
+        string = "".join(
+            # Use repr on the string literal parts
+            # to get proper escapes, e.g. \n, \\, \"
+            # But strip the quotes off the ends
+            # (they will always be one character: ' or ")
+            repr(value.value)[1:-1]
+            # Literal braces must be doubled to escape them
+            .replace("{", "{{").replace("}", "}}")
+            # Each value in values is either a string literal (Const)
+            # or a FormattedValue
+            if type(value).__name__ == "Const" else value.accept(self)
+            for value in node.values
+        )
+
+        # Try to find surrounding quotes that don't appear at all in the string.
+        # Because the formatted values inside {} can't contain backslash (\)
+        # using a triple quote is sometimes necessary
+        for quote in ["'", '"', '"""', "'''"]:
+            if quote not in string:
+                break
+
+        return "f" + quote + string + quote
+
+    def visit_formattedvalue(self, node):
+        result = node.value.accept(self)
+        if node.conversion and node.conversion >= 0:
+            # e.g. if node.conversion == 114: result += "!r"
+            result += "!" + chr(node.conversion)
+        if node.format_spec:
+            # The format spec is itself a JoinedString, i.e. an f-string
+            # We strip the f and quotes of the ends
+            result += ":" + node.format_spec.accept(self)[2:-1]
+        return "{%s}" % result
+
+    def handle_functiondef(self, node, keyword):
+        """return a (possibly async) function definition node as string"""
         decorate = node.decorators.accept(self) if node.decorators else ""
         docs = self._docs_dedent(node.doc) if node.doc else ""
         trailer = ":"
         if node.returns:
-            return_annotation = "->" + node.returns.as_string()
+            return_annotation = " -> " + node.returns.as_string()
             trailer = return_annotation + ":"
-        def_format = "\n%sdef %s(%s)%s%s\n%s"
+        def_format = "\n%s%s %s(%s)%s%s\n%s"
         return def_format % (
             decorate,
+            keyword,
             node.name,
             node.args.accept(self),
             trailer,
@@ -324,16 +365,26 @@ class AsStringVisitor:
             self._stmt_list(node.body),
         )
 
+    def visit_functiondef(self, node):
+        """return an astroid.FunctionDef node as string"""
+        return self.handle_functiondef(node, "def")
+
+    def visit_asyncfunctiondef(self, node):
+        """return an astroid.AsyncFunction node as string"""
+        return self.handle_functiondef(node, "async def")
+
     def visit_generatorexp(self, node):
         """return an astroid.GeneratorExp node as string"""
-        return "(%s %s)" % (
-            node.elt.accept(self),
-            " ".join(n.accept(self) for n in node.generators),
+        return "({} {})".format(
+            node.elt.accept(self), " ".join(n.accept(self) for n in node.generators)
         )
 
     def visit_attribute(self, node):
         """return an astroid.Getattr node as string"""
-        return "%s.%s" % (self._precedence_parens(node, node.expr), node.attrname)
+        left = self._precedence_parens(node, node.expr)
+        if left.isdigit():
+            left = "(%s)" % left
+        return f"{left}.{node.attrname}"
 
     def visit_global(self, node):
         """return an astroid.Global node as string"""
@@ -341,7 +392,7 @@ class AsStringVisitor:
 
     def visit_if(self, node):
         """return an astroid.If node as string"""
-        ifs = ["if %s:\n%s" % (node.test.accept(self), self._stmt_list(node.body))]
+        ifs = ["if {}:\n{}".format(node.test.accept(self), self._stmt_list(node.body))]
         if node.has_elif_block():
             ifs.append("el%s" % self._stmt_list(node.orelse, indent=False))
         elif node.orelse:
@@ -350,7 +401,7 @@ class AsStringVisitor:
 
     def visit_ifexp(self, node):
         """return an astroid.IfExp node as string"""
-        return "%s if %s else %s" % (
+        return "{} if {} else {}".format(
             self._precedence_parens(node, node.body, is_left=True),
             self._precedence_parens(node, node.test, is_left=True),
             self._precedence_parens(node, node.orelse, is_left=False),
@@ -364,14 +415,14 @@ class AsStringVisitor:
         """return an astroid.Keyword node as string"""
         if node.arg is None:
             return "**%s" % node.value.accept(self)
-        return "%s=%s" % (node.arg, node.value.accept(self))
+        return "{}={}".format(node.arg, node.value.accept(self))
 
     def visit_lambda(self, node):
         """return an astroid.Lambda node as string"""
         args = node.args.accept(self)
         body = node.body.accept(self)
         if args:
-            return "lambda %s: %s" % (args, body)
+            return f"lambda {args}: {body}"
 
         return "lambda: %s" % body
 
@@ -381,9 +432,8 @@ class AsStringVisitor:
 
     def visit_listcomp(self, node):
         """return an astroid.ListComp node as string"""
-        return "[%s %s]" % (
-            node.elt.accept(self),
-            " ".join(n.accept(self) for n in node.generators),
+        return "[{} {}]".format(
+            node.elt.accept(self), " ".join(n.accept(self) for n in node.generators)
         )
 
     def visit_module(self, node):
@@ -395,6 +445,16 @@ class AsStringVisitor:
         """return an astroid.Name node as string"""
         return node.name
 
+    def visit_namedexpr(self, node):
+        """Return an assignment expression node as string"""
+        target = node.target.accept(self)
+        value = node.value.accept(self)
+        return f"{target} := {value}"
+
+    def visit_nonlocal(self, node):
+        """return an astroid.Nonlocal node as string"""
+        return "nonlocal %s" % ", ".join(node.names)
+
     def visit_pass(self, node):
         """return an astroid.Pass node as string"""
         return "pass"
@@ -405,20 +465,16 @@ class AsStringVisitor:
         if not node.nl:
             nodes = "%s," % nodes
         if node.dest:
-            return "print >> %s, %s" % (node.dest.accept(self), nodes)
+            return "print >> {}, {}".format(node.dest.accept(self), nodes)
         return "print %s" % nodes
 
     def visit_raise(self, node):
         """return an astroid.Raise node as string"""
         if node.exc:
-            if node.inst:
-                if node.tback:
-                    return "raise %s, %s, %s" % (
-                        node.exc.accept(self),
-                        node.inst.accept(self),
-                        node.tback.accept(self),
-                    )
-                return "raise %s, %s" % (node.exc.accept(self), node.inst.accept(self))
+            if node.cause:
+                return "raise {} from {}".format(
+                    node.exc.accept(self), node.cause.accept(self)
+                )
             return "raise %s" % node.exc.accept(self)
         return "raise"
 
@@ -443,9 +499,8 @@ class AsStringVisitor:
 
     def visit_setcomp(self, node):
         """return an astroid.SetComp node as string"""
-        return "{%s %s}" % (
-            node.elt.accept(self),
-            " ".join(n.accept(self) for n in node.generators),
+        return "{{{} {}}}".format(
+            node.elt.accept(self), " ".join(n.accept(self) for n in node.generators)
         )
 
     def visit_slice(self, node):
@@ -454,8 +509,8 @@ class AsStringVisitor:
         upper = node.upper.accept(self) if node.upper else ""
         step = node.step.accept(self) if node.step else ""
         if step:
-            return "%s:%s:%s" % (lower, upper, step)
-        return "%s:%s" % (lower, upper)
+            return f"{lower}:{upper}:{step}"
+        return f"{lower}:{upper}"
 
     def visit_subscript(self, node):
         """return an astroid.Subscript node as string"""
@@ -467,7 +522,7 @@ class AsStringVisitor:
             # Remove parenthesis in tuple and extended slice.
             # a[(::1, 1:)] is not valid syntax.
             idxstr = idxstr[1:-1]
-        return "%s[%s]" % (self._precedence_parens(node, node.value), idxstr)
+        return "{}[{}]".format(self._precedence_parens(node, node.value), idxstr)
 
     def visit_tryexcept(self, node):
         """return an astroid.TryExcept node as string"""
@@ -480,9 +535,8 @@ class AsStringVisitor:
 
     def visit_tryfinally(self, node):
         """return an astroid.TryFinally node as string"""
-        return "try:\n%s\nfinally:\n%s" % (
-            self._stmt_list(node.body),
-            self._stmt_list(node.finalbody),
+        return "try:\n{}\nfinally:\n{}".format(
+            self._stmt_list(node.body), self._stmt_list(node.finalbody)
         )
 
     def visit_tuple(self, node):
@@ -497,13 +551,15 @@ class AsStringVisitor:
             operator = "not "
         else:
             operator = node.op
-        return "%s%s" % (operator, self._precedence_parens(node, node.operand))
+        return "{}{}".format(operator, self._precedence_parens(node, node.operand))
 
     def visit_while(self, node):
         """return an astroid.While node as string"""
-        whiles = "while %s:\n%s" % (node.test.accept(self), self._stmt_list(node.body))
+        whiles = "while {}:\n{}".format(
+            node.test.accept(self), self._stmt_list(node.body)
+        )
         if node.orelse:
-            whiles = "%s\nelse:\n%s" % (whiles, self._stmt_list(node.orelse))
+            whiles = "{}\nelse:\n{}".format(whiles, self._stmt_list(node.orelse))
         return whiles
 
     def visit_with(self, node):  # 'with' without 'as' is possible
@@ -512,7 +568,7 @@ class AsStringVisitor:
             ("%s" % expr.accept(self)) + (vars and " as %s" % (vars.accept(self)) or "")
             for expr, vars in node.items
         )
-        return "with %s:\n%s" % (items, self._stmt_list(node.body))
+        return "with {}:\n{}".format(items, self._stmt_list(node.body))
 
     def visit_yield(self, node):
         """yield an ast.Yield node as string"""
@@ -521,7 +577,16 @@ class AsStringVisitor:
         if node.parent.is_statement:
             return expr
 
-        return "(%s)" % (expr,)
+        return f"({expr})"
+
+    def visit_yieldfrom(self, node):
+        """ Return an astroid.YieldFrom node as string. """
+        yi_val = (" " + node.value.accept(self)) if node.value else ""
+        expr = "yield from" + yi_val
+        if node.parent.is_statement:
+            return expr
+
+        return f"({expr})"
 
     def visit_starred(self, node):
         """return Starred node as string"""
@@ -538,78 +603,11 @@ class AsStringVisitor:
     def visit_uninferable(self, node):
         return str(node)
 
+    def visit_property(self, node):
+        return node.function.accept(self)
 
-class AsStringVisitor3(AsStringVisitor):
-    """AsStringVisitor3 overwrites some AsStringVisitor methods"""
-
-    def visit_excepthandler(self, node):
-        if node.type:
-            if node.name:
-                excs = "except %s as %s" % (
-                    node.type.accept(self),
-                    node.name.accept(self),
-                )
-            else:
-                excs = "except %s" % node.type.accept(self)
-        else:
-            excs = "except"
-        return "%s:\n%s" % (excs, self._stmt_list(node.body))
-
-    def visit_nonlocal(self, node):
-        """return an astroid.Nonlocal node as string"""
-        return "nonlocal %s" % ", ".join(node.names)
-
-    def visit_raise(self, node):
-        """return an astroid.Raise node as string"""
-        if node.exc:
-            if node.cause:
-                return "raise %s from %s" % (
-                    node.exc.accept(self),
-                    node.cause.accept(self),
-                )
-            return "raise %s" % node.exc.accept(self)
-        return "raise"
-
-    def visit_yieldfrom(self, node):
-        """ Return an astroid.YieldFrom node as string. """
-        yi_val = (" " + node.value.accept(self)) if node.value else ""
-        expr = "yield from" + yi_val
-        if node.parent.is_statement:
-            return expr
-
-        return "(%s)" % (expr,)
-
-    def visit_asyncfunctiondef(self, node):
-        function = super(AsStringVisitor3, self).visit_functiondef(node)
-        return "async " + function.strip()
-
-    def visit_await(self, node):
-        return "await %s" % node.value.accept(self)
-
-    def visit_asyncwith(self, node):
-        return "async %s" % self.visit_with(node)
-
-    def visit_asyncfor(self, node):
-        return "async %s" % self.visit_for(node)
-
-    def visit_joinedstr(self, node):
-        # Special treatment for constants,
-        # as we want to join literals not reprs
-        string = "".join(
-            value.value if type(value).__name__ == "Const" else value.accept(self)
-            for value in node.values
-        )
-        return "f'%s'" % string
-
-    def visit_formattedvalue(self, node):
-        return "{%s}" % node.value.accept(self)
-
-    def visit_comprehension(self, node):
-        """return an astroid.Comprehension node as string"""
-        return "%s%s" % (
-            "async " if node.is_async else "",
-            super(AsStringVisitor3, self).visit_comprehension(node),
-        )
+    def visit_evaluatedobject(self, node):
+        return node.original.accept(self)
 
 
 def _import_string(names):
@@ -617,14 +615,11 @@ def _import_string(names):
     _names = []
     for name, asname in names:
         if asname is not None:
-            _names.append("%s as %s" % (name, asname))
+            _names.append(f"{name} as {asname}")
         else:
             _names.append(name)
     return ", ".join(_names)
 
 
-if sys.version_info >= (3, 0):
-    AsStringVisitor = AsStringVisitor3
-
 # This sets the default indent to 4 spaces.
 to_code = AsStringVisitor("    ")
